// - Spatial simple -
// This simple example creates an spatial handle, initializes it, hooks the event handlers and opens it.  It then waits
// for a spatial to be attached and waits for events to be fired. We preset the data rate to 16ms, but can be set higher (eg. 200)
// in order to slow down the events to make them more visible.
//
// Copyright 2010 Phidgets Inc.  All rights reserved.
// This work is licensed under the Creative Commons Attribution 2.5 Canada License. 
// view a copy of this license, visit http://creativecommons.org/licenses/by/2.5/ca/

#include "spatialfilter.h"

FILE *file = fopen("C:\\Users\\Brian Desktop\\Desktop\\spatial_data.csv","w");
int i = 0;
dataPt spatialData[250]; //data to be filtered
Engine *ep; //create a matlab engine object
bool warningFlag = false;

//callback that will run if the Spatial is attached to the computer
int CCONV AttachHandler(CPhidgetHandle spatial, void *userptr)
{
	int serialNo;
	CPhidget_getSerialNumber(spatial, &serialNo);
	printf("Spatial %10d attached!", serialNo);

	return 0;
}

//callback that will run if the Spatial is detached from the computer
int CCONV DetachHandler(CPhidgetHandle spatial, void *userptr)
{
	int serialNo;
	CPhidget_getSerialNumber(spatial, &serialNo);
	printf("Spatial %10d detached! \n", serialNo);

	return 0;
}

//callback that will run if the Spatial generates an error
int CCONV ErrorHandler(CPhidgetHandle spatial, void *userptr, int ErrorCode, const char *unknown)
{
	printf("Error handled. %d - %s \n", ErrorCode, unknown);
	return 0;
}

//callback that will run at datarate
//data - array of spatial event data structures that holds the spatial data packets that were sent in this event
//count - the number of spatial data event packets included in this event
int CCONV SpatialDataHandler(CPhidgetSpatialHandle spatial, void *userptr, CPhidgetSpatial_SpatialEventDataHandle *data, int count)
{
	if(i < SAMPLESIZE){
		for(int j=0;j<count;j++){//for each data packet in this event
			spatialData[i+j].timestamp.seconds = data[j]->timestamp.seconds;
			spatialData[i+j].timestamp.microseconds = data[j]->timestamp.microseconds;
			spatialData[i+j].xAccel = data[j]->acceleration[0]; 
			spatialData[i+j].yAccel = data[j]->acceleration[1];
			spatialData[i+j].zAccel = data[j]->acceleration[2];
		}
		i=i+count;
	}
	else{
		filterandprint(); //print data to file every SAMPLESIZE samples and also print filtered data
		i = 0;
	}
	if(i%100==0&&i%100<9)
		printf(".");
	if(!i)
		printf("|");
	fflush(stdout);
	return 0;
}

//Display the properties of the attached phidget to the screen.  
//We will be displaying the name, serial number, version of the attached device, the number of accelerometer, gyro, and compass Axes, and the current data rate
// of the attached Spatial.
int display_properties(CPhidgetHandle phid)
{
	int serialNo, version;
	const char* ptr;
	int numAccelAxes, numGyroAxes, numCompassAxes, dataRateMax, dataRateMin;

	CPhidget_getDeviceType(phid, &ptr);
	CPhidget_getSerialNumber(phid, &serialNo);
	CPhidget_getDeviceVersion(phid, &version);
	CPhidgetSpatial_getAccelerationAxisCount((CPhidgetSpatialHandle)phid, &numAccelAxes);
	CPhidgetSpatial_getGyroAxisCount((CPhidgetSpatialHandle)phid, &numGyroAxes);
	CPhidgetSpatial_getCompassAxisCount((CPhidgetSpatialHandle)phid, &numCompassAxes);
	CPhidgetSpatial_getDataRateMax((CPhidgetSpatialHandle)phid, &dataRateMax);
	CPhidgetSpatial_getDataRateMin((CPhidgetSpatialHandle)phid, &dataRateMin);

	

	printf("%s\n", ptr);
	printf("Serial Number: %10d\nVersion: %8d\n", serialNo, version);
	printf("Number of Accel Axes: %i\n", numAccelAxes);
	printf("Number of Gyro Axes: %i\n", numGyroAxes);
	printf("Number of Compass Axes: %i\n", numCompassAxes);
	printf("datarate> Max: %d  Min: %d\n", dataRateMax, dataRateMin);

	return 0;
}

int spatial_simple()
{
	int result;
	const char *err;
	
	spatialData->timestamp.microseconds=0;
	spatialData->timestamp.seconds=0;
	spatialData->xAccel=0;
	spatialData->yAccel=0;
	spatialData->zAccel=0;

	//Declare a spatial handle
	CPhidgetSpatialHandle spatial = 0;

	//create the spatial object
	CPhidgetSpatial_create(&spatial);

	//Set the handlers to be run when the device is plugged in or opened from software, unplugged or closed from software, or generates an error.
	CPhidget_set_OnAttach_Handler((CPhidgetHandle)spatial, AttachHandler, NULL);
	CPhidget_set_OnDetach_Handler((CPhidgetHandle)spatial, DetachHandler, NULL);
	CPhidget_set_OnError_Handler((CPhidgetHandle)spatial, ErrorHandler, NULL);

	//Registers a callback that will run according to the set data rate that will return the spatial data changes
	//Requires the handle for the Spatial, the callback handler function that will be called, 
	//and an arbitrary pointer that will be supplied to the callback function (may be NULL)
	CPhidgetSpatial_set_OnSpatialData_Handler(spatial, SpatialDataHandler, NULL);

	//open the spatial object for device connections
	CPhidget_open((CPhidgetHandle)spatial, -1);

	//get the program to wait for a spatial device to be attached
	printf("Waiting for spatial to be attached.... \n");
	if((result = CPhidget_waitForAttachment((CPhidgetHandle)spatial, 10000)))
	{
		CPhidget_getErrorDescription(result, &err);
		printf("Problem waiting for attachment: %s\n", err);
		return 0;
	}

	//Display the properties of the attached spatial device
	display_properties((CPhidgetHandle)spatial);

	//read spatial event data
	printf("Reading.....\n");
	
	//Set the data rate for the spatial events
	int maxdata;
	CPhidgetSpatial_getDataRateMax((CPhidgetSpatialHandle) spatial, &maxdata);
	CPhidgetSpatial_setDataRate((CPhidgetSpatialHandle)spatial, maxdata);

	//run until user input is read
	printf("Press any key to end\n");
	getchar();

	//since user input has been read, this is a signal to terminate the program so we will close the phidget and delete the object we created
	printf("Closing...\n");
	CPhidget_close((CPhidgetHandle)spatial);
	CPhidget_delete((CPhidgetHandle)spatial);

	return 0;
}

//main entry point to the program
int main(int argc, char* argv[])
{
	fprintf(file, "Time,Accel_X,Accel_Y,Accel_Z,Filtered_X,Filtered_Y,Filtered_Z\n"); //prep csv
	fflush(file);
	spatial_simple(); //lets get this show on the road
	fclose(file);
	return 0;
}



void filterandprint(){
	if (!(ep = engOpen(NULL))) { //open engine and quit if unsuccessful
		printf("Matlab engine failed to open, quitting.");
		exit(-1);
	}

	dataPt filtered[SAMPLESIZE];
	double x[SAMPLESIZE], y[SAMPLESIZE], z[SAMPLESIZE];
	double *xptr, *yptr, *zptr;
	int error = 0;
	for(int j=0;j<SAMPLESIZE;j++){
		x[j] = spatialData[j].xAccel;
		y[j] = spatialData[j].yAccel;
		z[j] = spatialData[j].zAccel;
	}


	//split into 3 arrays (one for each axis)
	mxArray *X = NULL, *Y = NULL, *Z = NULL;
	X = mxCreateDoubleMatrix(1, SAMPLESIZE, mxREAL);
	memcpy((char *) mxGetPr(X), (char *) x, SAMPLESIZE*sizeof(double));
	Y = mxCreateDoubleMatrix(1, SAMPLESIZE, mxREAL);
	memcpy((char *) mxGetPr(Y), (char *) y, SAMPLESIZE*sizeof(double));
	Z = mxCreateDoubleMatrix(1, SAMPLESIZE, mxREAL);
	memcpy((char *) mxGetPr(Z), (char *) z, SAMPLESIZE*sizeof(double));

	//send them to the engine
	error = error & engPutVariable(ep, "X", X);
	error = error & engPutVariable(ep, "Y", Y);
	error = error & engPutVariable(ep, "Z", Z);

	if(error != 0){
		printf("One or more of the variables could not be sent to Matlab\n");
		exit(EXIT_FAILURE);
	}


	//set up the filter
	engEvalString(ep, "[b,a]=butter(20,0.25,'high');");
	

	//apply filter to arrays
	error = 0;
	error = error & engEvalString(ep, "X = filter(b,a, X);");
	error = error & engEvalString(ep, "Y = filter(b,a, Y);");
	error = error & engEvalString(ep, "Z = filter(b,a, Z);");

	if(error != 0){
		printf("One or more of the Matlab commands did not complete successfully.  Are you sure the engine is still running?\n");
		exit(EXIT_FAILURE);
	}

	//get data back out of engine
	X = engGetVariable(ep, "X");
	Y = engGetVariable(ep, "Y");
	Z = engGetVariable(ep, "Z");
	if(X == NULL || Y == NULL || Z == NULL){
		printf("One or more of the variables could not be retrieved from Matlab.\n");
		exit(EXIT_FAILURE);
	}

	xptr = mxGetPr(X);
	yptr = mxGetPr(Y);
	zptr = mxGetPr(Z);
	for(int j=0;j<SAMPLESIZE;j++){
		filtered[j].xAccel = *xptr++;
		filtered[j].yAccel = *yptr++;
		filtered[j].zAccel = *zptr++;
		if(filtered[j].xAccel > FILTER_THRESHOLD || filtered[j].yAccel > FILTER_THRESHOLD || filtered[j].zAccel > FILTER_THRESHOLD)
			warningFlag = true;
	}
	notify();

	mxDestroyArray(X);
	mxDestroyArray(Y);
	mxDestroyArray(Z);


	//print data out to file
	for(int k=0;k<SAMPLESIZE;k++){
		// Timestamp, then Accel X,Y,Z Filtered X,Y,Z
		fprintf(file, "%f,",spatialData[k].timestamp.seconds + (spatialData[k].timestamp.microseconds)/1000000.0);
		fprintf(file, "%6f,%6f,%6f,%6f,%6f,%6f\n", spatialData[k].xAccel, spatialData[k].yAccel, spatialData[k].zAccel, filtered[k].xAccel, filtered[k].yAccel, filtered[k].zAccel);
		fflush(file);
		spatialData[k].timestamp.seconds=0;
		spatialData[k].timestamp.microseconds=0;
		spatialData[k].xAccel=0;
		spatialData[k].yAccel=0;
		spatialData[k].zAccel=0;
	}

}

void notify(){ //warn the user if there is rapid acceleration changes.
	if(warningFlag == false)
		return;
	else{
		printf("DANGER");
		warningFlag = false;
	}
}